/************************************************************************************
  Pointsw_r01.ino   LN Version
/************************************************************************************* 
   This sketch requires the following libraries:
    Loconet Library 1.1.4 

   2021/10/20 Rev01
   Copyright (c) 2021 Nagoden  All right reserved.
*************************************************************************************/
#include <EEPROM.h>

//----------------------------------------------------------------------
// Settings used in Loconet
#include <LocoNet.h>
#define THROWN 0            //Switch Dir Parameter
#define CLOSED 32           //Switch Dir Parameter
#define DAT_PIN0 A0         //input Sw PORT 1
#define DAT_PIN1 A1         //input Sw PORT 2
#define DAT_PIN2 A2         //input Sw PORT 3
#define DAT_PIN3 A3         //input Sw PORT 4
#define DAT_PIN4 3          //input Sw PORT 5
#define DAT_PIN5 4          //input Sw PORT 6
#define DAT_PIN6 5          //input Sw PORT 7
#define DAT_PIN7 6          //input Sw PORT 8
#define SETUP_PIN 9         //input SetupPin
#define STAT_LED_PIN 10      // Status_LED on/off pin

/*********************************************************************************
 Variable definition
 ********************************************************************************/
uint16_t  Startadress = 0;    //バイナリでのスイッチ開始アドレス
int       SetUpMode = 0;      //セットアップモードフラグ　セットアップモードで１
uint16_t  gSwitchAddress;     //スイッチコマンド受信アドレス
int       gSwitchDir;         //ポイント方向　 0:t:分岐  32:c:直線
int       ValueSwitch[8] ;    //8個のスイッチの状態保持変数
int       ValuePort[8] ;      //8個のポート状態保持変数
boolean   gBlinkflg = false;  //LED点滅フラグ
/*********************************************************************************
 Loconet Class定義
 ********************************************************************************/
LocoNetClass  Loconet;
lnMsg       *LnPacket;
LnBuf       LnTxBuffer;
/*********************************************************************************
 Task Schedule
 ********************************************************************************/
unsigned long gPreviousL1 = 0; //   300ms interval  Adress setup mode LED Blink Cicle
unsigned long gPreviousL2 = 0; //   Not Used
unsigned long gPreviousL3 = 0; //   Not Used

/*********************************************************************************
 SETUP
 ********************************************************************************/
void setup() {
    Serial.begin(115200);
    //Serial.println("Start");
  /*First initialize the LocoNet interface*/ 
      LocoNet.init(7);
  //EEPROMからアドレス読み出し
      EEPROM.get(0x000, Startadress);
      if ((Startadress > 0x270F) || (Startadress == 0 ) ) {
          //EEPROMの値が9999より大きいか０の場合はアドレスを初期値0にする
          Startadress = 0 ;
      }
    //Serial.print("EEPROM_R:");
    //Serial.println(Startadress);

  //LED初期化
      pinMode(DAT_PIN0 , INPUT_PULLUP);  // SwitchPin1
      pinMode(DAT_PIN1 , INPUT_PULLUP);  // SwitchPin2
      pinMode(DAT_PIN2 , INPUT_PULLUP);  // SwitchPin3
      pinMode(DAT_PIN3 , INPUT_PULLUP);  // SwitchPin4
      pinMode(DAT_PIN4 , INPUT_PULLUP);  // SwitchPin5
      pinMode(DAT_PIN5 , INPUT_PULLUP);  // SwitchPin6
      pinMode(DAT_PIN6 , INPUT_PULLUP);  // SwitchPin7
      pinMode(DAT_PIN7 , INPUT_PULLUP);  // SwitchPin8
      pinMode(SETUP_PIN , INPUT_PULLUP);  // Setup Pin
      pinMode(STAT_LED_PIN , OUTPUT);  // Power LED
      digitalWrite(STAT_LED_PIN, HIGH);
  //Reset task
      gPreviousL1 = millis();
      gPreviousL2 = millis();
      gPreviousL3 = millis();

  //セットアップモード判定 
      if (digitalRead(SETUP_PIN) == 0 ){
         SetUpMode = 1;
      }     
      else{
        SetUpMode = 0;
        gPreviousL1 = 0;
      }
}
 

/*********************************************************************************
  Loop処理
 ********************************************************************************/

void loop() {
 /*********************************************
  * Loconet受信処理
  *********************************************/
  // Check for any received LocoNet packets
  LnPacket = LocoNet.receive() ;
  if (LnPacket)  {
      //Serial.println("LnPacket");
      if (!LocoNet.processSwitchSensorMessage(LnPacket)){
        digitalWrite(STAT_LED_PIN, LOW);
        delay(10);
        digitalWrite(STAT_LED_PIN, HIGH);
      }
  }

  if (SetUpMode == 0) {
    //通常モード
       //ポートの状態を読み取り 
       ValuePort[0]= digitalRead(DAT_PIN0);
       ValuePort[1]= digitalRead(DAT_PIN1);
       ValuePort[2]= digitalRead(DAT_PIN2);
       ValuePort[3]= digitalRead(DAT_PIN3);
       ValuePort[4]= digitalRead(DAT_PIN4);
       ValuePort[5]= digitalRead(DAT_PIN5);
       ValuePort[6]= digitalRead(DAT_PIN6);
       ValuePort[7]= digitalRead(DAT_PIN7);
      //スイッチの状態が変化したらスイッチコマンドを出力      
       for (int i=0; i <= 7; i++){
          if ( ValuePort[i] !=  ValueSwitch[i]){
              ValueSwitch[i] = ValuePort[i];
              if (ValueSwitch[i] == 0){
                  Loconet.requestSwitch(Startadress+i,1,THROWN);
              }
              else{
                  Loconet.requestSwitch(Startadress+i,1,CLOSED);
              }
              delay(200);
          }
       }
  }
  else{
    //セットアップモード　
      LED_Blink();
      //スイッチで”ｃ”が押されたらそのスイッチのアドレスをEEPROMに書き込んで通常モードに変移
      if (gSwitchDir == CLOSED) {
          AddressEEPROM_Write ();
          delay(100);
          SetUpMode = 0 ; 
          gPreviousL1 = 0;
          Startadress = gSwitchAddress;     
      }
      gSwitchAddress = 0;
      gSwitchDir = 0;
    
  }

}

/***********************************************************************
  各種関数
************************************************************************/
//EEPROMへのアドレス登録
void AddressEEPROM_Write (void) {
    EEPROM.put(0x000, gSwitchAddress);
//  Serial.print("EEPROM_W:");
//  Serial.println(gSwitchAddress);
}

//ＬＥＤ点滅
void LED_Blink(void){
  if ( (millis() - gPreviousL1) >= 300)  {
      //Reset task
      gPreviousL1 = millis();
      if (gBlinkflg == false){
         digitalWrite(STAT_LED_PIN, HIGH);
         gBlinkflg = true ;
      }   
      else{
         digitalWrite(STAT_LED_PIN, LOW);
         gBlinkflg = false ;
      } 
  }   
}
//以下はLocoNetでのコールバック関数
//--------------------------------------------------------------------------------------------------
//Notifications for accessory control requests
//--------------------------------------------------------------------------------------------------
void notifySwitchRequest( uint16_t Address, uint8_t outData, uint8_t dirData) {
//  Serial.print("notifySwitchRequest():");
//  Serial.print(Address,DEC);
//  Serial.print(",");
//  Serial.print(outData,DEC); 
//  Serial.print(",");
//  Serial.print(dirData,DEC); // 0:t:分岐  32:c:直線
//  Serial.println("");
  gSwitchAddress =  Address;
  gSwitchDir = dirData;
}

//--------------------------------------------------------------------------------------------------
